package geometry.part1;

import java.awt.geom.Point2D;

/**
 * 直線 a * x + b * y + c = 0 を表現するクラス
 */
public class Line {
    public double a;
    public double b;
    public double c;

    public Line(double a, double b, double c) {
        this.a = a;
        this.b = b;
        this.c = c;
    }

    public static Line fromPoints(double x1, double y1, double x2, double y2) {
        double dx = x2 - x1;
        double dy = y2 - y1;
        return new Line(dy, -dx, dx * y1 - dy * x1);
    }

    public static Line perpendicularBisector(double x1, double y1, double x2,
            double y2) {
        double cx = (x1 + x2) / 2.0;
        double cy = (y1 + y2) / 2.0;
        return fromPoints(cx, cy, cx + (y1 - y2), cy + (x2 - x1));
    }

    public static Line perpendicularBisector(Point2D p1, Point2D p2) {
        return perpendicularBisector(p1.getX(), p1.getY(), p2.getX(), p2.getY());
    }

    public Point2D getIntersectionPoint(Line l) {
        double d = a * l.b - l.a * b;
        if (d == 0.0) {
            return null; // 直線が平行の場合はnullを返す
        }
        double x = (b * l.c - l.b * c) / d;
        double y = (l.a * c - a * l.c) / d;
        return new Point2D.Double(x, y);
    }

    @Override
    public String toString() {
        return "a = " + a + ", b = " + b + ", c = " + c;
    }
}
