package geometry.part12;

import geometry.part1.Line;
import geometry.part10.PolygonIntersectionCalculator;
import geometry.part6.ConvexPolygon;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.List;

public class VoronoiGenerator {
    private PseudoHalfPlaneGenerator halfPlaneGenerator = new PseudoHalfPlaneGenerator(
            10); // この値を大きくしすぎると計算誤差が無視できなくなる
    private PolygonIntersectionCalculator intersectionCalculator = new PolygonIntersectionCalculator();

    // areaの領域内で、母点sitesの各ボロノイ領域をリストにして返す。
    // areaおよびsitesの各座標が、halfPlaneGeneratorの境界内に収まるように注意すること
    public List<ConvexPolygon> execute(ConvexPolygon area, List<Point2D> sites) {
        List<ConvexPolygon> result = new ArrayList<ConvexPolygon>();
        for (Point2D s1 : sites) {
            ConvexPolygon region = null; // 途中計算結果格納用の領域
            for (Point2D s2 : sites) {
                if (s1 == s2) {
                    continue;
                }
                // s1とs2の垂直二等分線を求める
                Line line = Line.perpendicularBisector(s1, s2);
                // 垂直二等分線による半平面のうち、s1を含む方を求める
                ConvexPolygon halfPlane = halfPlaneGenerator.execute(line, s1);
                if (region == null) { // 初回計算時なら
                    // areaと半平面の交差を求める
                    region = intersectionCalculator.execute(area, halfPlane);
                } else { // 2回目以降なら
                    // これまでの計算結果と半平面の交差を求める
                    region = intersectionCalculator.execute(region, halfPlane);
                }
            }
            // 最終的な計算結果をボロノイ領域とする
            result.add(region);
        }
        return result;
    }
}
