package geometry.part2;

import geometry.part1.Line;
import geometry.part6.GeomUtils;

import java.awt.geom.Point2D;

/**
 * 2点(x1, y1) - (x2, y2)を結ぶ線分を表現するクラス
 */
public class LineSegment {
    public double x1;
    public double y1;
    public double x2;
    public double y2;

    public LineSegment(double x1, double y1, double x2, double y2) {
        this.x1 = x1;
        this.y1 = y1;
        this.x2 = x2;
        this.y2 = y2;
    }

    public LineSegment(Point2D p1, Point2D p2) {
        this(p1.getX(), p1.getY(), p2.getX(), p2.getY());
    }

    public Line toLine() {
        return Line.fromPoints(x1, y1, x2, y2);
    }

    public boolean intersects(Line l) {
        double t1 = l.a * x1 + l.b * y1 + l.c;
        double t2 = l.a * x2 + l.b * y2 + l.c;
        return t1 * t2 <= 0;
    }

    // 第2回の実装（sと自線分が一直線上にある場合に誤動作）
    // public boolean intersects(LineSegment s) {
    // return intersects(s.toLine()) && s.intersects(toLine());
    // }

    // 第7回で書き直し
    public boolean intersects(LineSegment s) {
        return bothSides(s) && s.bothSides(this);
    }

    // sが自線分の「両側」にあるかどうかを調べる
    private boolean bothSides(LineSegment s) {
        double ccw1 = GeomUtils.ccw(x1, y1, s.x1, s.y1, x2, y2);
        double ccw2 = GeomUtils.ccw(x1, y1, s.x2, s.y2, x2, y2);
        if (ccw1 == 0 && ccw2 == 0) { // sと自線分が一直線上にある場合
            // sのいずれか1つの端点が自線分を内分していれば、sは自線分と共有部分を持つので
            // trueを返す
            return internal(s.x1, s.y1) || internal(s.x2, s.y2);
        } else { // それ以外の場合
            // CCW値の符号が異なる場合にtrueを返す
            return ccw1 * ccw2 <= 0;
        }
    }

    // (x, y)が自線分を内分しているかどうかを調べる
    private boolean internal(double x, double y) {
        // (x, y)から端点に向かうベクトルの内積がゼロ以下であれば内分と見なす
        return GeomUtils.dot(x1 - x, y1 - y, x2 - x, y2 - y) <= 0;
    }

    public Point2D getIntersectionPoint(Line l) {
        if (!intersects(l)) {
            return null; // 交差しない場合はnullを返す
        }
        return l.getIntersectionPoint(toLine());
    }

    public Point2D getIntersectionPoint(LineSegment s) {
        if (!intersects(s)) {
            return null; // 交差しない場合はnullを返す
        }
        return s.toLine().getIntersectionPoint(toLine());
    }

    @Override
    public String toString() {
        return "(" + x1 + ", " + y1 + ") - (" + x2 + ", " + y2 + ")";
    }
}
