package geometry.part2;

import geometry.common.DemoFrame;
import geometry.common.DemoPanel;
import geometry.common.Demo;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

public class LineSegmentDemo {
    private enum Focus {
        S1_1, S1_2, S2_1, S2_2
    }

    public static void main(String[] args) {
        Demo.launch(LineSegmentFrame.class, "線分の交差");
    }

    public static class LineSegmentFrame extends DemoFrame {
        private static final long serialVersionUID = 1L;

        LineSegment s1;
        LineSegment s2;
        Focus focus;

        @Override
        protected void initialize() {
            s1 = new LineSegment(100, 150, 400, 450);
            s2 = new LineSegment(120, 350, 460, 200);

            final LineSegmentPanel panel = new LineSegmentPanel();
            add(panel, BorderLayout.CENTER);

            panel.addMouseMotionListener(new MouseAdapter() {
                @Override
                public void mouseMoved(MouseEvent e) {
                    double x = e.getX();
                    double y = e.getY();
                    Focus befFocus = focus;
                    if (Math.abs(s1.x1 - x) + Math.abs(s1.y1 - y) <= 8) {
                        focus = Focus.S1_1;
                    } else if (Math.abs(s1.x2 - x) + Math.abs(s1.y2 - y) <= 8) {
                        focus = Focus.S1_2;
                    } else if (Math.abs(s2.x1 - x) + Math.abs(s2.y1 - y) <= 8) {
                        focus = Focus.S2_1;
                    } else if (Math.abs(s2.x2 - x) + Math.abs(s2.y2 - y) <= 8) {
                        focus = Focus.S2_2;
                    } else {
                        focus = null;
                    }
                    if (focus != befFocus) {
                        panel.repaint();
                    }
                }

                @Override
                public void mouseDragged(MouseEvent e) {
                    double x = e.getX();
                    double y = e.getY();
                    if (focus == Focus.S1_1) {
                        s1.x1 = x;
                        s1.y1 = y;
                    } else if (focus == Focus.S1_2) {
                        s1.x2 = x;
                        s1.y2 = y;
                    } else if (focus == Focus.S2_1) {
                        s2.x1 = x;
                        s2.y1 = y;
                    } else if (focus == Focus.S2_2) {
                        s2.x2 = x;
                        s2.y2 = y;
                    }
                    panel.repaint();
                }
            });
        }

        private class LineSegmentPanel extends DemoPanel {
            private static final long serialVersionUID = 1L;

            @Override
            protected void doPaint(Graphics2D g) {
                g.setPaint(Color.BLACK);
                drawLineSegment(g, s1, focus == Focus.S1_1, focus == Focus.S1_2);
                drawLineSegment(g, s2, focus == Focus.S2_1, focus == Focus.S2_2);

                Point2D p = s1.getIntersectionPoint(s2);
                if (p != null) {
                    g.setPaint(Color.RED);
                    g.fill(new Ellipse2D.Double(p.getX() - 8, p.getY() - 8, 16,
                            16));
                }
            }

            private void drawLineSegment(Graphics2D g, LineSegment s,
                    boolean focus1, boolean focus2) {
                g.draw(new Line2D.Double(s.x1, s.y1, s.x2, s.y2));
                double r1 = focus1 ? 8 : 4;
                double r2 = focus2 ? 8 : 4;
                g.fill(new Ellipse2D.Double(s.x1 - r1, s.y1 - r1, 2 * r1,
                        2 * r1));
                g.fill(new Ellipse2D.Double(s.x2 - r2, s.y2 - r2, 2 * r2,
                        2 * r2));
            }
        }
    }
}
